/*  ********************************************************************
	TESTBLMS.C
	Test application for the BLMS filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptblms32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
#define LMS		// uncomment ONLY one of the following algorithms
//#define SGNERR
//#define SGNREG
//#define SGNSGN

#define ITERATIONS 5000
#define NC 10		/* filter length		 */
#define NL 5		/* block length          */
#define HL 8        /* unknown filter length */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptBlms32f  blms ;			/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		Nc = NC;			/* filter length		    */
	int		Nl = NL;			/* block length             */
	float	mu = 0.1f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	float	inpBuf[4*NL];
	float	desBuf[4*NL];
	float	errBuf[4*NL];
	float	outBuf[4*NL];
	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	float	fir[4*NC];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCof[NC];
	float	pDat[NC]; 	
#endif


	/* create the unknown filter */
	i = fir32fInit(&hFir, HL, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, 4*NL);
	reset32f(desBuf, 4*NL);
	reset32f(outBuf, 4*NL);
	reset32f(errBuf, 4*NL);	

	/* Create and initialize the asptBlms filter */
#ifndef STATIC
		i = blms32fInit(&blms,Nc,Nl,mu);
#else
		i = blms32fInitStatic(&blms,Nc,Nl,mu,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptBlms filter\n");
		exit(-1);
	}

#ifdef FILTERONLY
	j= blms32fSetCoef(&blms, unknown, 0, NC,0);
	printf("Coef set = %d\n",j);
	blms32fGetCoef(&blms, fir, 0, NC, 0);
	dispR32f(fir,j);	
#endif

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", errFileName); 
		return(-3);
	}

	start = clock();
	duration = (double) 0.0;

	for (i=0; i< (ITERATIONS-Nl);i+=Nl)	{

		/* fill in the input block */
		for (k=0; k<Nl; k++){		
			inpBuf[k] = (float) rand()/(float)RAND_MAX - 0.5f;;	
		}

		/* desired signal */
		fir32fBlockFilter(&hFir, inpBuf, desBuf, Nl);


#ifdef FILTERONLY
		if (i < 400*Nl)	{	
			/* filter but no update */
			blms32fFilterOnly(&blms, inpBuf, outBuf);
			sub32f(outBuf, desBuf, errBuf, Nl);
		}
		else
#endif
		{
			/* filter and update the filter coefficients */



#ifdef LMS
			blms32fFilterUpdate(&blms, inpBuf, desBuf, outBuf,errBuf);
#endif
#ifdef SGNERR
			blms32fSetStepSize(&blms,0.01f);
			blms32fFilterUpdateSignErr(&blms, inpBuf, desBuf, outBuf,errBuf);
#endif
#ifdef SGNREG
			blms32fFilterUpdateSignReg(&blms, inpBuf, desBuf, outBuf,errBuf);
#endif
#ifdef SGNSGN
			blms32fSetStepSize(&blms,0.01f);
			blms32fFilterUpdateSignSign(&blms, inpBuf, desBuf, outBuf,errBuf);
#endif

		}

		/* write samples to streams */
		for (j=0; j<Nl;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}

#ifdef RESET
		if (i == 500*Nl)	{
			blms32fReset(&blms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 500 * Nl){
			Nc = 8;
			Nl = 7;
			if (Nl<NL)
			   i+= (NL-Nl);
			j = blms32fResize(&blms, Nc,Nl);
			printf("RESIZE returned %d\n",j);
			j = blms32fGetFilterLength(&blms);
			k = blms32fGetCoef(&blms, fir, 0, j, 0);
			dispR32f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	//copy32f(blms.wFreq,fir,FL);
	//dispR32f(fir, FL);
	blms32fGetCoef(&blms, fir,0, Nc,0);
	dispR32f(fir, blms32fGetFilterLength(&blms));
	fdispC32f(firFile, fir, blms32fGetFilterLength(&blms));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, BL=%d, SS=%f, coefPtr=%d,\n", 
	blms32fGetFilterLength(&blms),  
	blms32fGetBlockLength(&blms), 
	blms32fGetStepSize(&blms),
	blms32fGetCoefPtr(&blms));

#ifndef STATIC
	blms32fResize(&blms,12,4);
#endif

	blms32fSetStepSize(&blms, 0.111f);
//	blms32fSetBlockLength(&blms, 12);

	printf("FL=%d, BL=%d, SS=%f, coefPtr=%d,\n", 
	blms32fGetFilterLength(&blms),  
	blms32fGetBlockLength(&blms), 
	blms32fGetStepSize(&blms),
	blms32fGetCoefPtr(&blms));
#endif

#ifndef STATIC
	blms32fFree(&blms);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "BLMS              %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );


	return 0;
}



