/*  ********************************************************************
	TESTLMSLATTICE.C 
	simple system identification test application for the LMSLATTICE 
	adaptive algorithm.
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#include <time.h>
#include "asptlmslattice32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED

#define ITERATIONS 5000
#define NF 8
#define NH 8

int main(int argc, char* argv[])
{

	asptLmslattice32f  lmsl ;	/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		len = 8;	// filter length
	float	mu_p = 0.001f;
	float	mu_c = 0.01f;
	int		i,j=0, k=0;

	FILE	*outFile;			/* out signal log file	*/
	FILE	*errFile;			/* err signal log file	*/
	FILE	*inpFile;			/* inp signal log file	*/
	FILE	*desFile;			/* des signal log file	*/

	float	inpBuf[ITERATIONS];
	float	desBuf[ITERATIONS];
	float	errBuf[ITERATIONS];
	float	outBuf[ITERATIONS];

	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};
	float	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;		/* FIR coef. file */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCof[NF*2];		/* 2*filterLength */
	float	pDat[NF*3];		/* 3*filterLength */	
#endif

	/* create the unknown filter */
	i = fir32fInit(&hFir, NH, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, ITERATIONS);
	reset32f(desBuf, ITERATIONS);
	reset32f(outBuf, ITERATIONS);
	reset32f(errBuf, ITERATIONS);	

	/* Create and initialize the asptLmslattice filter */
#ifndef STATIC
		i = lmslattice32fInit(&lmsl,len,mu_p, mu_c);
#else
		i = lmslattice32fInitStatic(&lmsl,len,mu_p, mu_c,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptLmslattice filter\n");
		exit(-1);
	}

	// make a random inp signal
	for (i=0; i<ITERATIONS; i++)
		inpBuf[i] = (float) rand()/(float)RAND_MAX - 0.5f;

	/* calculate the des signal */
	fir32fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

#ifdef FILTERONLY
	i = lmslattice32fSetCoef(&lmsl, unknown, 0, len, 0);
	printf("coef copied = %d\n",i);
	dispR32f(lmsl.coef,i);
	lmslattice32fSetParcorUpdateFlag(&lmsl,0);
#endif

	start = clock();
	duration = (double) 0.0;
	

	for (i=0; i<ITERATIONS;i++)	{

#ifdef FILTERONLY
		if (i < 1500){
			outBuf[i] = lmslattice32fFilterOnly(&lmsl, inpBuf[i]);
			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{	// filter and update the filter coefficient
			outBuf[i] = lmslattice32fFilterUpdate(&lmsl, inpBuf[i], desBuf[i]);
			errBuf[i] = lmslattice32fGetLastErrorSample(&lmsl);

		}
#ifdef RESET
		if (i == 3200){
			lmslattice32fReset(&lmsl);
		}
#endif
#ifdef RESIZE
#ifndef STATIC
		if (i == 3000)	{// resize
			len = 10;
			j = lmslattice32fResize(&lmsl, len);
			j = lmslattice32fGetCoef(&lmsl, fir, 0, len, 0);
			dispR32f(fir, j);
		}
#endif
#endif
		if (i==1500){
			lmslattice32fSetParcorUpdateFlag(&lmsl,0);
		}
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}
#ifndef SPEED
		/* write samples to streams */
		for (j=0; j<ITERATIONS;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
#endif
	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	// save and print the filter coefficients	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	i = lmslattice32fGetCoef(&lmsl, fir, 0, len, 0);
	fdispR32f(firFile,fir,len);
	dispR32f(fir,len);
	fclose(firFile);

#endif

#ifdef MACROS

	printf("L=%d, S1=%f, S2=%f, F=%d, DLPtr=%d, PARCORPtr=%d, CofPtr=%d\n", 
		lmslattice32fGetFilterLength(&lmsl),  
		lmslattice32fGetPcStepSize(&lmsl), 
		lmslattice32fGetLcStepSize(&lmsl),
		lmslattice32fGetParcorUpdateFlag(&lmsl),
		lmslattice32fGetDelayLinePtr(&lmsl),
		lmslattice32fGetParcorPtr(&lmsl),
		lmslattice32fGetCoefPtr(&lmsl));

#ifndef STATIC
	lmslattice32fResize(&lmsl,12);
#endif
	lmslattice32fSetPcStepSize(&lmsl, 0.111f);
	lmslattice32fSetLcStepSize(&lmsl, 2*0.111f);
	lmslattice32fSetParcorUpdateFlag(&lmsl, 2);

	printf("L=%d, S1=%f, S2=%f, F=%d, DLPtr=%d, PARCORPtr=%d, CofPtr=%d\n", 
		lmslattice32fGetFilterLength(&lmsl),  
		lmslattice32fGetPcStepSize(&lmsl), 
		lmslattice32fGetLcStepSize(&lmsl),
		lmslattice32fGetParcorUpdateFlag(&lmsl),
		lmslattice32fGetDelayLinePtr(&lmsl),
		lmslattice32fGetParcorPtr(&lmsl),
		lmslattice32fGetCoefPtr(&lmsl));
#endif

#ifndef STATIC
	lmslattice32fFree(&lmsl);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "LMSLATTICE       %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );

	return 0;
}


