/*  ********************************************************************
	TESTNLMS.C
	Test application for the NLMS filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptnlms32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED
//#define LC

#define ITERATIONS 5000
#define NF 10		/* adaptive filter length */
#define HL 8        /* unknown filter length  */

int main(int argc, char* argv[])
{

	int		i,k,j=0;
	asptNlms32f  nlms ;			/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		Nf = NF;			/* filter length		    */
	float	mu = 0.1f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	float	inpBuf[ITERATIONS];
	float	desBuf[ITERATIONS];
	float	errBuf[ITERATIONS];
	float	outBuf[ITERATIONS];
	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	float	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCof[NF];		/* filterLength */
	float	pDat[NF];		/* filterLength */	
#endif


	/* create the unknown filter */
	i = fir32fInit(&hFir, HL, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, ITERATIONS);
	reset32f(desBuf, ITERATIONS);
	reset32f(outBuf, ITERATIONS);
	reset32f(errBuf, ITERATIONS);	

	/* Create and initialize the asptNlms filter */
#ifndef STATIC
		i = nlms32fInit(&nlms,Nf,mu);
#else
		i = nlms32fInitStatic(&nlms,Nf,mu,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptNlms filter\n");
		exit(-1);
	}


#ifdef FILTERONLY
	j= nlms32fSetCoef(&nlms, unknown, 0, Nf,0);
	printf("%d coef have been set.\n",j);
	j = nlms32fGetCoef(&nlms, fir, 0, Nf, 0);
	dispR32f(fir,j);	
#endif

	/* fill in the input buffer */
	for (k=0; k<ITERATIONS; k++){		
		inpBuf[k] = (float) rand()/(float)RAND_MAX - 0.5f;;	
	}

	/* calculate the desired signal */
	fir32fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{
#ifdef LC
		insertSampleLinBuf32f(inpBuf[i],nlms.delayLine,nlms32fGetDelayLineLength(&nlms));
#endif

#ifdef FILTERONLY
		if (i < 2000)	{	
			/* filter but no update */
#ifdef LC
			outBuf[i] = nlms32fLCFilterOnly(&nlms);
#else
			outBuf[i] = nlms32fFilterOnly(&nlms, inpBuf[i]);
#endif

			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
#ifdef LC
			outBuf[i] =	nlms32fLCFilterUpdate(&nlms, desBuf[i]);
#else
			outBuf[i] =	nlms32fFilterUpdate(&nlms, inpBuf[i], desBuf[i]);
#endif

			errBuf[i] = nlms32fGetLastErrorSample(&nlms);
		}



#ifdef RESET
		if (i == 3000)	{
			nlms32fReset(&nlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			Nf = 8;
			j = nlms32fResize(&nlms, Nf);
			printf("RESIZE returned %d\n RESIZE::",j);
			j = nlms32fGetFilterLength(&nlms);
			k = nlms32fGetCoef(&nlms, fir, 0, j, 0);
			dispR32f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", errFileName); 
		return(-3);
	}
#ifndef SPEED
		/* write samples to streams */
		for (i=0; i<ITERATIONS;i++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[i]);
			fprintf(desFile, "%+6.5f\n", desBuf[i]);
			fprintf(outFile, "%+6.5f\n", outBuf[i]);		
			fprintf(errFile, "%+6.5f\n", errBuf[i]);
		}
#endif

	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	dispR32f(nlms.coef, Nf);
	nlms32fGetCoef(&nlms, fir,0, Nf,0);
	dispR32f(fir, nlms32fGetFilterLength(&nlms));
	fdispC32f(firFile, fir, nlms32fGetFilterLength(&nlms));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, SS=%f, coefPtr=%d,\n", 
	nlms32fGetFilterLength(&nlms),  
	nlms32fGetStepSize(&nlms),
	nlms32fGetCoefPtr(&nlms));

#ifndef STATIC
	nlms32fResize(&nlms,12);
#endif

	nlms32fSetStepSize(&nlms, 0.111f);

	printf("FL=%d, SS=%f, coefPtr=%d,\n", 
	nlms32fGetFilterLength(&nlms),  
	nlms32fGetStepSize(&nlms),
	nlms32fGetCoefPtr(&nlms));
#endif

#ifndef STATIC
	nlms32fFree(&nlms);
#endif
	fir32fFree(&hFir);

   printf("\nElapsed Time      in Seconds\n");
   printf(  "NLMS              %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );


	return 0;
}



