/*  ********************************************************************
	TESTPBFDAF.C
	Test application for the PBFDAF filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptpbfdaf32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"
#include "rfft16.h"
#include "rfft8.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS

#define ITERATIONS 5000
#define NC 6		/* controller length     */
#define NP 2		/* partitions            */
#define NL 6		/* block length          */
#define FL 16       /* FFT length            */
#define HL 12       /* unknown filter length */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptPbfdaf32f  pbfdaf ;		/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		Nc = NC;			/* filter length		    */
	int		Nl = NL;			/* block length             */
	int		Np = NP;			/* number of partitions     */
	int		cs = 1;				/* use constrained BFDAF    */
	float	mu = 0.05f;			/* step size                */
	int		Length = Nc * Np;	/* total filter length      */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	float	inpBuf[2*NL];
	float	desBuf[2*NL];
	float	errBuf[2*NL];
	float	outBuf[2*NL];
	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f, .1f, .4f, .8f, .6f};

	float	fir[20*NC];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	int		*ip = ip16;			/* twiddle factor table          */
	float	*w  = w16;          /* bit reversal table            */
	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCoef[NP*FL];						/* [NP*fftLength]                          */
	float	pDat[(FL-NL)+(NP+2)*FL+((FL>>1)+1)]; /* [C1+(NP+2)*fftLength+(fftLength/2+1)] */	
#endif


	/* create the unknown filter */
	i = fir32fInit(&hFir, HL, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, 2*NL);
	reset32f(desBuf, 2*NL);
	reset32f(outBuf, 2*NL);
	reset32f(errBuf, 2*NL);	

	/* Create and initialize the asptBfdaf filter */
#ifndef STATIC
		i = pbfdaf32fInit(&pbfdaf,Nc,Nl,Np,mu,cs, ip,w);
#else
		i = pbfdaf32fInitStatic(&pbfdaf,Nc,Nl,Np,mu,cs,pCoef,pDat,ip,w);
#endif	
	
	if (i < 0){
		printf("Error creating asptPbfdaf filter\n");
		exit(-1);
	}
/*
	printf("FilterLength = %d\n", pbfdaf.filterLength);
	printf("BlockLength  = %d\n", pbfdaf.blockLength);
	printf("Partitions   = %d\n", pbfdaf.partitions);
	printf("FFTLength    = %d\n", pbfdaf.fftLength);
	printf("C1           = %d\n", pbfdaf.C1);
*/
#ifdef FILTERONLY
	j= pbfdaf32fSetCoef(&pbfdaf, unknown, 0, Length,0);
	printf("Coef set = %d\n",j);
	pbfdaf32fGetCoef(&pbfdaf, fir, 0, Length, 0);
	dispR32f(fir,j);	
#endif

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", errFileName); 
		return(-3);
	}

	start = clock();
	duration = (double) 0.0;

	for (i=0; i< (ITERATIONS-Nl);i+=Nl)	{

		/* fill in the input block */
		for (k=0; k<Nl; k++){		
			inpBuf[k] = (float) rand()/(float)RAND_MAX - 0.5f;;	
		}

		/* desired signal */
		fir32fBlockFilter(&hFir, inpBuf, desBuf, Nl);


#ifdef FILTERONLY
		if (i < 400*Nl)	{	
			/* filter but no update */
			pbfdaf32fFilterOnly(&pbfdaf, inpBuf, outBuf);
			sub32f(outBuf, desBuf, errBuf, Nl);
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			pbfdaf32fFilterUpdate(&pbfdaf, inpBuf, desBuf, outBuf,errBuf);
		}

		/* write samples to streams */
		for (j=0; j<Nl;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}

#ifdef RESET
		if (i == 500*Nl)	{
			pbfdaf32fReset(&pbfdaf);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 500 * Nl){
			Nc = 7;
			Nl = 7;
			Np = 2;
			Length = Np * Nc;
			if (Nl<NL)
			   i+= (NL-Nl);
			j = pbfdaf32fResize(&pbfdaf, Nc,Nl,Np);
			printf("RESIZE returned %d\n",j);
			j = pbfdaf32fGetFilterLength(&pbfdaf);
			k = pbfdaf32fGetCoef(&pbfdaf, fir, 0, j*Np, 0);
			dispR32f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	pbfdaf32fGetCoef(&pbfdaf, fir,0, Length,0);
	dispR32f(fir, Length);
	fdispC32f(firFile, fir, Length);

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, BL=%d, FFTL=%d, SS=%f, CS=%d, coefPtr=%d, partitions=%d\n", 
	pbfdaf32fGetFilterLength(&pbfdaf),  
	pbfdaf32fGetBlockLength(&pbfdaf), 
	pbfdaf32fGetFftLength(&pbfdaf),
	pbfdaf32fGetStepSize(&pbfdaf),
	pbfdaf32fGetConstrain(&pbfdaf), 
	pbfdaf32fGetCoefPtr(&pbfdaf),
	pbfdaf32fGetPartitions(&pbfdaf)	);

#ifndef STATIC
	pbfdaf32fResize(&pbfdaf,5,4,4);
#endif

	pbfdaf32fSetStepSize(&pbfdaf, 0.111f);
	pbfdaf32fSetConstrain(&pbfdaf, 0);

	printf("FL=%d, BL=%d, FFTL=%d, SS=%f, CS=%d, coefPtr=%d, partitions=%d\n", 
	pbfdaf32fGetFilterLength(&pbfdaf),  
	pbfdaf32fGetBlockLength(&pbfdaf), 
	pbfdaf32fGetFftLength(&pbfdaf),
	pbfdaf32fGetStepSize(&pbfdaf),
	pbfdaf32fGetConstrain(&pbfdaf), 
	pbfdaf32fGetCoefPtr(&pbfdaf),
	pbfdaf32fGetPartitions(&pbfdaf)	);
#endif

#ifndef STATIC
	pbfdaf32fFree(&pbfdaf);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "PBFDAF            %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );


	return 0;
}



