/*  ********************************************************************
	TESTRLSLATTICE.C 
	simple system identification test application for the RLS LATTICE 
	adaptive algorithm.
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#include <time.h>
#include "asptrlslattice32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED

#define ITERATIONS 5000
#define NF 8
#define NH 8

int main(int argc, char* argv[])
{

	asptRlslattice32f  rlsl ;	/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		len = 8;	// filter length
	float	mu = 0.998f;
	int		i,j=0, k=0;

	FILE	*outFile;			/* out signal log file	*/
	FILE	*errFile;			/* err signal log file	*/
	FILE	*inpFile;			/* inp signal log file	*/
	FILE	*desFile;			/* des signal log file	*/

	float	inpBuf[ITERATIONS];
	float	desBuf[ITERATIONS];
	float	errBuf[ITERATIONS];
	float	outBuf[ITERATIONS];

	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};
	float	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;		/* FIR coef. file */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCof[NF*3];		/* 3*filterLength */
	float	pDat[NF*6];		/* 6*filterLength */	
#endif

	/* create the unknown filter */
	i = fir32fInit(&hFir, NH, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, ITERATIONS);
	reset32f(desBuf, ITERATIONS);
	reset32f(outBuf, ITERATIONS);
	reset32f(errBuf, ITERATIONS);	

	/* Create and initialize the asptRlslattice filter */
#ifndef STATIC
		i = rlslattice32fInit(&rlsl,len,mu);
#else
		i = rlslattice32fInitStatic(&rlsl,len,mu,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptRlslattice filter\n");
		exit(-1);
	}

	// make a random inp signal
	for (i=0; i<ITERATIONS; i++)
		inpBuf[i] = (float) rand()/(float)RAND_MAX - 0.5f;

	/* calculate the des signal */
	fir32fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

	start = clock();
	duration = (double) 0.0;
	

	for (i=0; i<ITERATIONS;i++)	{

#ifdef FILTERONLY
		if (i >= 1500){
			outBuf[i] = rlslattice32fFilterOnly(&rlsl, inpBuf[i]);
			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{	// filter and update the filter coefficient
			outBuf[i] = rlslattice32fFilterUpdate(&rlsl, inpBuf[i], desBuf[i]);
			errBuf[i] = rlslattice32fGetLastErrorSample(&rlsl);

		}
#ifdef RESET
		if (i == 3200){
			rlslattice32fReset(&rlsl);
		}
#endif
#ifdef RESIZE
#ifndef STATIC
		if (i == 3000)	{// resize
			len = 10;
			j = rlslattice32fResize(&rlsl, len);
			j = rlslattice32fGetCoef(&rlsl, fir, 0, len, 0);
			dispR32f(fir, j);
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}
#ifndef SPEED
		/* write samples to streams */
		for (j=0; j<ITERATIONS;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
#endif
	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	// save and print the filter coefficients	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	i = rlslattice32fGetCoef(&rlsl, fir, 0, len, 0);
	printf("LINEAR COMBINER::\n");
	fdispR32f(firFile,fir,len);
	dispR32f(fir,len);

	i = rlslattice32fGetForwardParcors(&rlsl, fir, 0, len, 0);
	printf("FORWARD PARKOR::\n");
	fdispR32f(firFile,fir,len);
	dispR32f(fir,len);

	i = rlslattice32fGetBackwardParcors(&rlsl, fir, 0, len, 0);
	printf("BACKWARD PARKOR::\n");
	fdispR32f(firFile,fir,len);
	dispR32f(fir,len);

	fclose(firFile);

#endif

#ifdef MACROS

	printf("L=%d, FF=%f, DLPtr=%d, BP=%d, FP=%d, CofPtr=%d\n", 
		rlslattice32fGetFilterLength(&rlsl),  
		rlslattice32fGetForgetFactor(&rlsl), 
		rlslattice32fGetDelayLinePtr(&rlsl),
		rlslattice32fGetBackwardParcorPtr(&rlsl),
		rlslattice32fGetForwardParcorPtr(&rlsl),
		rlslattice32fGetCoefPtr(&rlsl));

#ifndef STATIC
	rlslattice32fResize(&rlsl,12);
#endif

	rlslattice32fSetForgetFactor(&rlsl, 0);

	printf("L=%d, FF=%f, DLPtr=%d, BP=%d, FP=%d, CofPtr=%d\n", 
		rlslattice32fGetFilterLength(&rlsl),  
		rlslattice32fGetForgetFactor(&rlsl), 
		rlslattice32fGetDelayLinePtr(&rlsl),
		rlslattice32fGetBackwardParcorPtr(&rlsl),
		rlslattice32fGetForwardParcorPtr(&rlsl),
		rlslattice32fGetCoefPtr(&rlsl));
#endif

#ifndef STATIC
	rlslattice32fFree(&rlsl);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "RLSLATTICE       %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );

	return 0;
}


