/*  ********************************************************************
	TESTTDLMS.C
	Test application for the TDLMS filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "aspttdlms32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"
#include "rfft16.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
#define LC

#define ITERATIONS 5000
#define NF 10		/* adaptive filter length */
#define HL 8        /* unknown filter length  */
#define FL 16       /* FFT length            */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptTdlms32f  tdlms ;		/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		Nf = NF;			/* filter length		    */
	float	mu = 0.1f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	float	inpBuf[ITERATIONS];
	float	desBuf[ITERATIONS];
	float	errBuf[ITERATIONS];
	float	outBuf[ITERATIONS];
	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	float	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	int		*ip = ip16;			/* twiddle factor table     */
	float	*w  = w16;           /* bit reversal table       */

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file			*/	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCof[FL];
	float	pDat[3*FL+2];		
#endif


	/* create the unknown filter */
	i = fir32fInit(&hFir, HL, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, ITERATIONS);
	reset32f(desBuf, ITERATIONS);
	reset32f(outBuf, ITERATIONS);
	reset32f(errBuf, ITERATIONS);	

	/* Create and initialize the asptBfdaf filter */
#ifndef STATIC
		i = tdlms32fInit(&tdlms,Nf,mu, ip, w);
#else
		i = tdlms32fInitStatic(&tdlms,Nf,mu,pCof,pDat,ip,w);
#endif	
	
	if (i < 0){
		printf("Error creating asptNlms filter\n");
		exit(-1);
	}


#ifdef FILTERONLY
	j= tdlms32fSetCoef(&tdlms, unknown, 0, Nf,0);
	printf("%d coef have been set.\n",j);
	j = tdlms32fGetCoef(&tdlms, fir, 0, Nf, 0);
	dispR32f(fir,j);	
#endif

	/* fill in the input buffer */
	for (k=0; k<ITERATIONS; k++){		
		inpBuf[k] = (float) rand()/(float)RAND_MAX - 0.5f;;	
	}

	/* calculate the desired signal */
	fir32fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{

#ifdef LC
		insertSampleLinBuf32f(inpBuf[i],tdlms.xTime,tdlms32fGetDelayLineLength(&tdlms));
#endif

#ifdef FILTERONLY
		if (i < 2000)	{	
			/* filter but no update */
#ifdef LC
			outBuf[i] = tdlms32fLCFilterOnly(&tdlms);
#else
			outBuf[i] = tdlms32fFilterOnly(&tdlms, inpBuf[i]);
#endif

			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
#ifdef LC
			outBuf[i] =	tdlms32fLCFilterUpdate(&tdlms, desBuf[i]);
#else
			outBuf[i] =	tdlms32fFilterUpdate(&tdlms, inpBuf[i], desBuf[i]);
#endif

			errBuf[i] = tdlms32fGetLastErrorSample(&tdlms);
		}



#ifdef RESET
		if (i == 3000)	{
			tdlms32fReset(&tdlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			Nf = 12;
			j = tdlms32fResize(&tdlms, Nf);
			printf("RESIZE returned %d\n RESIZE::",j);
			j = tdlms32fGetFilterLength(&tdlms);
			k = tdlms32fGetCoef(&tdlms, fir, 0, j, 0);
			dispR32f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}

		/* write samples to streams */
		for (j=0; j<ITERATIONS;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	tdlms32fGetCoef(&tdlms, fir,0, Nf,0);
	dispR32f(fir, tdlms32fGetFilterLength(&tdlms));
	fdispC32f(firFile, fir, tdlms32fGetFilterLength(&tdlms));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, FFTL=%d, SS=%f, coefPtr=%d,\n", 
	tdlms32fGetFilterLength(&tdlms),  
	tdlms32fGetFftLength(&tdlms),  
	tdlms32fGetStepSize(&tdlms),
	tdlms32fGetCoefPtr(&tdlms));

#ifndef STATIC
	tdlms32fResize(&tdlms,14);
#endif

	tdlms32fSetStepSize(&tdlms, 0.111f);

	printf("FL=%d, FFTL=%d, SS=%f, coefPtr=%d,\n", 
	tdlms32fGetFilterLength(&tdlms),  
	tdlms32fGetFftLength(&tdlms),  
	tdlms32fGetStepSize(&tdlms),
	tdlms32fGetCoefPtr(&tdlms));

#endif

#ifndef STATIC
	tdlms32fFree(&tdlms);
#endif
	fir32fFree(&hFir);

   printf("\nElapsed Time      in Seconds\n");
   printf(  "TDLMS             %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );


	return 0;
}



