/*  ********************************************************************
    TESTADJLMS.C
    Test application for ADJLMS
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */
#include <time.h>
#include "asptadjlms64f.h"
#include "asptfir64f.h"
#include "asptdisplay64f.h"


/****************/
/* Test Options */
/****************/
//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
//#define DELAYLINE
#define PRINTCOEF
//#define MACROS

#define ITERATIONS 5000

int main(int argc, char* argv[])
{

	asptAdjlms64f  adjlms ;
	int		len = 10;					/* controller length	       */
	asptFir64f pFir;					/* FIR filter for prim signal  */
	double	mu  = 0.01f ;				/* step size			       */
	int		i,j,k=0;
	FILE	*senFile;					/* sensor signal log file      */
	FILE	*actFile;					/* sec. source signal log file */
	double	ph[] = {0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};	/* Primary path impulse response	*/
	double	sh[] = {-1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};/* Secondary path impulse response	*/
	double	se[10];						/* estimation of sh	*/
	double	reference[ITERATIONS];		/* input reference signal					*/
	double	primary[ITERATIONS];		/* primary source signal					*/
	double	actBuf[ITERATIONS];		/* secondary source signal at the sensor	*/
	double	senBuf[ITERATIONS];		/* secondary source signal at the sensor	*/
	int		phLen = 10;
	int		shLen = 10;
	int		sheLen= 7;
	int		idx = 0;
	char	*senFileName="sen.txt";
	char	*actFileName="sec.txt";
	char	*firFileName="coef.txt";
   clock_t	start, finish;
   double	duration;

#ifdef PRINTCOEF
	FILE	*firFile;					/* file pointer for dumping FIR coef.*/
	double	*coef;
	double	fir[20];					/* buffer to print the controller coef.*/
#endif

#ifdef DELAYLINE
#ifndef PRINTCOEF
	double	fir[20];					/* buffer to print the delay line samples */
#endif
#endif

#ifdef STATIC
	double	pDat[10+10+2*7-1];
	double	pCof[10];
#endif

	/* make a random input signal */
	for (i=0; i<ITERATIONS; i++)
		reference[i] = (double) rand()/(double)RAND_MAX - 0.5f;

	j = MIN(shLen, sheLen);
	/* Estimate the secondary path */
	for (i=0; i<j; i++)	{
		se[i] = 1.0f * sh[i];
	}

	/* reset the buffers; */
	reset64f(actBuf, ITERATIONS);
	reset64f(senBuf, ITERATIONS);

	i = fir64fInit(&pFir, phLen, ph);

	/* calculate the primary signal */
	fir64fBlockFilter(&pFir, reference, primary, ITERATIONS);

	/* Create and initialize the ADJLMS */
#ifndef STATIC
		i = adjlms64fInit(&adjlms,len,mu,sh,shLen,se,sheLen);
#else
		i = adjlms64fInitStatic(&adjlms,len,mu,sh,shLen,se,sheLen,pCof,pDat);
#endif
		
	if (adjlms64fGetFilterLength(&adjlms) == 0){
		printf("ERROR %d : Error creating ADJLMS filter\n",i);
		exit(-1);
	}

#ifdef FILTERONLY
	i = adjlms64fSetCoef(&adjlms, ph, 0, len, 0);
	printf("coef copied = %d\n",i);
	dispR64f(adjlms.coef, i);
#endif

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{
		
#ifdef FILTERONLY
		if (i < 1500){
			/* filter only */
			actBuf[i] = adjlms64fFilterOnly(&adjlms, reference[i]);
			senBuf[i]  = primary[i] + actBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			actBuf[i] = adjlms64fFilterUpdate(&adjlms, reference[i], primary[i]);
			senBuf[i]  = adjlms64fGetLastSensorSample(&adjlms);
		}

#ifdef RESET
		if (i == 2000){
			/* reset */
			adjlms64fReset(&adjlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			/* resize */
			j = adjlms64fResize(&adjlms, 11);
			if (j != 0) {
				printf("Couldn't resize\n");
				exit(-1);
			}
			dispR64f(adjlms.coef, adjlms64fGetFilterLength(&adjlms));
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	actFile	= fopen (actFileName, "wt") ;
	if (actFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",actFileName); 
		return(-2);
	}

	senFile	= fopen (senFileName, "wt") ;
	if (senFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", senFileName); 
		return(-3);
	}
	
	/* write output and error samples to streams */
	for (i=0; i<ITERATIONS;i++)	{
		fprintf(senFile, "%f\n", senBuf[i]);
		fprintf(actFile, "%f\n", actBuf[i]);
	}
	fclose(senFile);
	fclose(actFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	coef = adjlms64fGetCoefPtr(&adjlms);
	i = adjlms64fGetCoef(&adjlms, fir, 0, len, 0);

	dispR64f(coef, adjlms64fGetFilterLength(&adjlms));
	//fdispR64f(firFile, fir, adjlms64fGetFilterLength(&adjlms));

	for (i=0; i<len; i++)
		fprintf(firFile, "%f  ", fir[i]);

	fclose(firFile);
#endif


#ifdef DELAYLINE
	/* test delay line Set and Get */
	adjlms64fGetDelayLine(&adjlms, fir, 0, 10, 0);
	dispR64f(fir, adjlms64fGetFilterLength(&adjlms));
//	adjlms64fResetDelayLine(&adjlms);
	adjlms64fGetDelayLine(&adjlms, fir, 0, 10, 0);
	dispR64f(fir, adjlms64fGetFilterLength(&adjlms));

	i = adjlms64fSetDelayLine(&adjlms, ph, 0, 5, 0);

	i = adjlms64fGetDelayLine(&adjlms, fir, 0, 10, 0);
	dispR64f(fir, adjlms64fGetFilterLength(&adjlms));

#endif

#ifdef MACROS
	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		adjlms64fGetFilterLength(&adjlms),  
		adjlms64fGetStepSize(&adjlms),
		adjlms64fGetCoefPtr(&adjlms),
		adjlms64fGetSecPathPtr(&adjlms),
		adjlms64fGetEstSecPathPtr(&adjlms),
		adjlms64fGetSecPathLength(&adjlms),
		adjlms64fGetEstSecPathLength(&adjlms) );

	adjlms64fSetStepSize(&adjlms, 0.111f);
	adjlms64fSetSecPathPtr(&adjlms, NULL);
	adjlms64fSetEstSecPathPtr(&adjlms, NULL);
	adjlms64fSetSecPathLength(&adjlms, 0);
	adjlms64fSetEstSecPathLength(&adjlms, 0);

	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		adjlms64fGetFilterLength(&adjlms),  
		adjlms64fGetStepSize(&adjlms),
		adjlms64fGetCoefPtr(&adjlms),
		adjlms64fGetSecPathPtr(&adjlms),
		adjlms64fGetEstSecPathPtr(&adjlms),
		adjlms64fGetSecPathLength(&adjlms),
		adjlms64fGetEstSecPathLength(&adjlms) );
#endif

  printf(  "Filter Length = %d\n",  adjlms64fGetFilterLength(&adjlms));
#ifndef STATIC
	adjlms64fFree(&adjlms);
#endif

  printf("\nElapsed Time    in Seconds\n");
  printf(  "ADJLMS          %10.10f\n",  (double)duration/ CLOCKS_PER_SEC );

	return 0;
}


