/*  ********************************************************************
	TESTBNLMS.C
	Test application for the BNLMS filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptbnlms64f.h"
#include "asptfir64f.h"
#include "asptdisplay64f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS

#define ITERATIONS 5000
#define NC 10		/* filter length		 */
#define NL 5		/* block length          */
#define HL 8        /* unknown filter length */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptBnlms64f  bnlms ;		/* adaptive filter          */
	asptFir64f hFir;			/* unknown filter           */
	int		Nc = NC;			/* filter length		    */
	int		Nl = NL;			/* block length             */
	double	mu = 0.01f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	double	inpBuf[4*NL];
	double	desBuf[4*NL];
	double	errBuf[4*NL];
	double	outBuf[4*NL];
	double	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	double	fir[4*NC];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	double	pCof[NC];
	double	pDat[NC]; 	
#endif


	/* create the unknown filter */
	i = fir64fInit(&hFir, HL, unknown);
	if (fir64fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset64f(inpBuf, 4*NL);
	reset64f(desBuf, 4*NL);
	reset64f(outBuf, 4*NL);
	reset64f(errBuf, 4*NL);	

	/* Create and initialize the asptBnlms filter */
#ifndef STATIC
		i = bnlms64fInit(&bnlms,Nc,Nl,mu);
#else
		i = bnlms64fInitStatic(&bnlms,Nc,Nl,mu,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptBnlms filter\n");
		exit(-1);
	}

#ifdef FILTERONLY
	j= bnlms64fSetCoef(&bnlms, unknown, 0, NC,0);
	printf("Coef set = %d\n",j);
	bnlms64fGetCoef(&bnlms, fir, 0, NC, 0);
	dispR64f(fir,j);	
#endif

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", errFileName); 
		return(-3);
	}

	start = clock();
	duration = (double) 0.0;

	for (i=0; i< (ITERATIONS-Nl);i+=Nl)	{

		/* fill in the input block */
		for (k=0; k<Nl; k++){		
			inpBuf[k] = (double) rand()/(double)RAND_MAX - 0.5f;;	
		}

		/* desired signal */
		fir64fBlockFilter(&hFir, inpBuf, desBuf, Nl);


#ifdef FILTERONLY
		if (i < 400*Nl)	{	
			/* filter but no update */
			bnlms64fFilterOnly(&bnlms, inpBuf, outBuf);
			sub64f(outBuf, desBuf, errBuf, Nl);
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			bnlms64fFilterUpdate(&bnlms, inpBuf, desBuf, outBuf,errBuf);
		}

		/* write samples to streams */
		for (j=0; j<Nl;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}

#ifdef RESET
		if (i == 500*Nl)	{
			bnlms64fReset(&bnlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 500 * Nl){
			Nc = 8;
			Nl = 7;
			if (Nl<NL)
			   i+= (NL-Nl);
			j = bnlms64fResize(&bnlms, Nc,Nl);
			printf("RESIZE returned %d\n",j);
			j = bnlms64fGetFilterLength(&bnlms);
			k = bnlms64fGetCoef(&bnlms, fir, 0, j, 0);
			dispR64f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	//copy64f(bnlms.wFreq,fir,FL);
	//dispR64f(fir, FL);
	bnlms64fGetCoef(&bnlms, fir,0, Nc,0);
	dispR64f(fir, bnlms64fGetFilterLength(&bnlms));
	fdispC64f(firFile, fir, bnlms64fGetFilterLength(&bnlms));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, BL=%d, SS=%f, coefPtr=%d,\n", 
	bnlms64fGetFilterLength(&bnlms),  
	bnlms64fGetBlockLength(&bnlms), 
	bnlms64fGetStepSize(&bnlms),
	bnlms64fGetCoefPtr(&bnlms));

#ifndef STATIC
	bnlms64fResize(&bnlms,12,4);
#endif

	bnlms64fSetStepSize(&bnlms, 0.111f);
//	bnlms64fSetBlockLength(&bnlms, 12);

	printf("FL=%d, BL=%d, SS=%f, coefPtr=%d,\n", 
	bnlms64fGetFilterLength(&bnlms),  
	bnlms64fGetBlockLength(&bnlms), 
	bnlms64fGetStepSize(&bnlms),
	bnlms64fGetCoefPtr(&bnlms));
#endif

#ifndef STATIC
	bnlms64fFree(&bnlms);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "BNLMS             %10.10f\n",  (double)duration/ CLOCKS_PER_SEC );


	return 0;
}



