/*  ********************************************************************
    TESTFXLMS.C
    Test application for FXLMS
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#include <time.h>
#include "asptfxlms64f.h"
#include "asptfir64f.h"
#include "asptdisplay64f.h"


/****************/
/* Test Options */
/****************/
//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
//#define DELAYLINE
#define PRINTCOEF
//#define MACROS

#define ITERATIONS 30000

int main(int argc, char* argv[])
{

	asptFxlms64f  fxlms ;
	int		len = 10;					/* controller length	       */
	asptFir64f pFir;					/* FIR filter for prim signal  */
	double	mu  = 0.01f ;				/* step size			       */
	int		i,j,k=0;
	FILE	*senFile;					/* sensor signal log file      */
	FILE	*actFile;					/* sec. source signal log file */
	FILE	*primFile;					/* primary signal log file     */
	double	ph[] = {0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};	/* Primary path impulse response	*/
	double	sh[] = {-1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};/* Secondary path impulse response	*/
	double	se[10];						/* estimation of sh	*/
	double	reference[ITERATIONS];		/* input reference signal					*/
	double	primary[ITERATIONS];		/* primary source signal					*/
	double	actBuf[ITERATIONS];			/* secondary source signal at the sensor	*/
	double	senBuf[ITERATIONS];			/* secondary source signal at the sensor	*/
	int		phLen = 10;
	int		shLen = 10;
	int		sheLen= 7;
	int		idx = 0;
	char	*senFileName="sen.txt";
	char	*actFileName="sec.txt";
	char	*firFileName="coef.txt";
	char	*primFileName="prim.txt";
   clock_t	start, finish;
   double	duration;

#ifdef PRINTCOEF
	FILE	*firFile;					/* file pointer for dumping FIR coef.*/
	double	*coef;
	double	fir[20];					/* buffer to print the controller coef.*/
#endif

#ifdef DELAYLINE
#ifndef PRINTCOEF
	double	fir[20];					/* buffer to print the delay line samples */
#endif
#endif

#ifdef STATIC
	/* MAX(L,seL)+sL+L */
	double	pDat[10+10+10];
	double	pCof[10];
#endif

	/* make a random input signal */
	for (i=0; i<ITERATIONS; i++)
		reference[i] = (double) rand()/(double)RAND_MAX - 0.5f;

	j = MIN(shLen, sheLen);
	/* Estimate the secondary path */
	for (i=0; i<j; i++)	{
		se[i] = 1.0f * sh[i];
	}

	/* reset the buffers; */
	reset64f(actBuf, ITERATIONS);
	reset64f(senBuf, ITERATIONS);

	i = fir64fInit(&pFir, phLen, ph);

	/* calculate the primary signal */
	fir64fBlockFilter(&pFir, reference, primary, ITERATIONS);

	/* Create and initialize the ADJLMS */
#ifndef STATIC
		i = fxlms64fInit(&fxlms,len,mu,sh,shLen,se,sheLen);
#else
		i = fxlms64fInitStatic(&fxlms,len,mu,sh,shLen,se,sheLen,pCof,pDat);
#endif
		
	if (fxlms64fGetFilterLength(&fxlms) == 0){
		printf("Error creating ADJLMS filter\n");
		exit(-1);
	}

#ifdef FILTERONLY
	i = fxlms64fSetCoef(&fxlms, ph, 0, len, 0);
	printf("coef copied = %d\n",i);
	dispR64f(fxlms.coef, i);
#endif

	duration = (double) 0.0;
	start = clock();

	for (i=0; i<ITERATIONS;i++)	{
		
#ifdef FILTERONLY
		if (i < 1500){
			/* filter only */
			actBuf[i] = fxlms64fFilterOnly(&fxlms, reference[i]);
			senBuf[i] = primary[i] + actBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			actBuf[i] = fxlms64fFilterUpdate(&fxlms, reference[i], primary[i]);
			senBuf[i] = fxlms64fGetLastSensorSample(&fxlms);
		}

#ifdef RESET
		if (i == 2000){
			/* reset */
			fxlms64fReset(&fxlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			/* resize */
			j = fxlms64fResize(&fxlms, 8);
			if (j != 0) {
				printf("Couldn't resize\n");
				exit(-1);
			}
			dispR64f(fxlms.coef, fxlms64fGetFilterLength(&fxlms));
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	actFile	= fopen (actFileName, "wt") ;
	if (actFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",actFileName); 
		return(-2);
	}

	senFile	= fopen (senFileName, "wt") ;
	if (senFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", senFileName); 
		return(-3);
	}
	
	primFile	= fopen (primFileName, "wt") ;
	if (senFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", primFileName); 
		return(-3);
	}

	/* write output and error samples to streams */
	for (i=0; i<ITERATIONS;i++)	{
		fprintf(senFile, "%f\n", senBuf[i]);
		fprintf(actFile, "%f\n", actBuf[i]);
		fprintf(primFile, "%f\n", primary[i]);
	}
	fclose(senFile);
	fclose(actFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	coef = fxlms64fGetCoefPtr(&fxlms);
	i = fxlms64fGetCoef(&fxlms, fir, 0, len, 0);

	dispR64f(coef, fxlms64fGetFilterLength(&fxlms));
	fdispR64f(firFile, fir, fxlms64fGetFilterLength(&fxlms));

	fclose(firFile);
#endif


#ifdef DELAYLINE
	/* test delay line Set and Get */
	fxlms64fGetDelayLine(&fxlms, fir, 0, 10, 0);
	dispR64f(fir, fxlms64fGetFilterLength(&fxlms));
//	fxlms64fResetDelayLine(&fxlms);
	fxlms64fGetDelayLine(&fxlms, fir, 0, 10, 0);
	dispR64f(fir, fxlms64fGetFilterLength(&fxlms));

	i = fxlms64fSetDelayLine(&fxlms, ph, 0, 5, 0);

	i = fxlms64fGetDelayLine(&fxlms, fir, 0, 10, 0);
	dispR64f(fir, fxlms64fGetFilterLength(&fxlms));

#endif

#ifdef MACROS
	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		fxlms64fGetFilterLength(&fxlms),  
		fxlms64fGetStepSize(&fxlms),
		fxlms64fGetCoefPtr(&fxlms),
		fxlms64fGetSecPathPtr(&fxlms),
		fxlms64fGetEstSecPathPtr(&fxlms),
		fxlms64fGetSecPathLength(&fxlms),
		fxlms64fGetEstSecPathLength(&fxlms) );

	fxlms64fSetStepSize(&fxlms, 0.111f);
	fxlms64fSetSecPathPtr(&fxlms, NULL);
	fxlms64fSetEstSecPathPtr(&fxlms, NULL);
	fxlms64fSetSecPathLength(&fxlms, 0);
	fxlms64fSetEstSecPathLength(&fxlms, 0);

	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		fxlms64fGetFilterLength(&fxlms),  
		fxlms64fGetStepSize(&fxlms),
		fxlms64fGetCoefPtr(&fxlms),
		fxlms64fGetSecPathPtr(&fxlms),
		fxlms64fGetEstSecPathPtr(&fxlms),
		fxlms64fGetSecPathLength(&fxlms),
		fxlms64fGetEstSecPathLength(&fxlms) );
#endif


#ifndef STATIC
	fxlms64fFree(&fxlms);
#endif

  printf("\nElapsed Time   ip processor time     in Seconds\n");
  printf(  "FXLMS         %10.6f            %10.6f\n",  duration, (double)duration/ CLOCKS_PER_SEC );

	return 0;
}


