/*  ********************************************************************
	TESTLBPEF.C 
	simple system identification test application for the LBPEF 
	adaptive algorithm.
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#include <time.h>
#include "asptdisplay64f.h"
#include "asptlbpef64f.h"



/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED

#define ITERATIONS 5000
#define NF 8

int main(int argc, char* argv[])
{
	asptLbpef64f	lbpef ;		// adaptive iir
	int		len = NF;			// filter length
	double	mu_p = 0.001f;
	int		i, j=0, k=0;
	double	fir[10];
	double	time ;
	double	fs	 = 1000.0f;		// sampling frequency
	double	ts   = 1/fs;		// sampling period

	FILE	*outFile;	/* output signal log file */
	FILE	*errFile;	/* error signal log file */

	double	inputBuf[ITERATIONS];
	double	errorBuf[ITERATIONS];
	double	outputBuf[ITERATIONS];

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file   */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	double	pCof[NF*2];			/* 2*filterLength */
	double	pDat[(NF+1)*4];		/* 4*(filterLength+1) */	
#endif
	

	// make input signal = a random input signal + sinusoidal of 50 Hz sampled at 1 kHz
	for (i=0, time = 0; i<ITERATIONS; i++)
	{ 
		time += ts;
		inputBuf[i] = (double) 2.0f * (rand()/(double)RAND_MAX - 0.5f);
		inputBuf[i] += (double) (1.0 * cos(PIX2 * 50.0 *time));
		inputBuf[i] += (double) (1.0 * cos(PIX2 * 200.0 *time));
		outputBuf[i] = 0.0f;
		errorBuf[i]  = 0.0f;
	}


	/* create lbpef filter */
#ifndef STATIC
		i = lbpef64fInit(&lbpef, len, mu_p);	
#else
		i = lbpef64fInitStatic(&lbpef, len, mu_p, pCof, pDat);
#endif	
	if (lbpef64fGetFilterLength(&lbpef) == 0)
	{
		printf("Error creating asptLbpef filter\n");
		exit(-1);
	}

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{

		
#ifdef FILTERONLY
		if (i > 1000)	// one second
		{
			outputBuf[i] = lbpef64fFilterOnly(&lbpef, inputBuf[i]);
			errorBuf[i]  = inputBuf[i] - outputBuf[i];
		}
		else
#endif
		{// filter and update the filter coefficients
			outputBuf[i] = lbpef64fFilterUpdate(&lbpef, inputBuf[i]);
			errorBuf[i]  = lbpef64fGetLastErrorSample(&lbpef);

		}

#ifdef RESET
		if (i == 4900){
			lbpef64fReset(&lbpef);
		}
#endif
#ifdef RESIZE
#ifndef STATIC
		if (i == 3000)	{// resize
			len = 10;
			j = lbpef64fResize(&lbpef, 10);
			j = lbpef64fGetCoef(&lbpef, fir, 0, len, 0);
			dispR64f(fir, j);
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	// Open files
	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)
	{
		fprintf(stdout,"Can't open file %s for writing",outFileName); 
		return(-2);
	}

	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)
	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}
#ifndef SPEED	
	// write output and error samples to streams
	for (i=0; i<ITERATIONS;i++)
	{
		fprintf(errFile, "%f\n", errorBuf[i]);
		fprintf(outFile, "%f\n", outputBuf[i]);
	}
#endif

	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL)
	{
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	i = lbpef64fGetCoef(&lbpef, fir, 0, len, 0);

	fdispR64f(firFile,fir,len);
	dispR64f(fir,len);

	fclose(firFile);
#endif

#ifdef MACROS

	printf("L=%d, S1=%f,  F=%d, DLPtr=%d, PARCORPtr=%d, CofPtr=%d\n", 
		lbpef64fGetFilterLength(&lbpef),  
		lbpef64fGetPcStepSize(&lbpef), 
		lbpef64fGetFirFlag(&lbpef),
		lbpef64fGetDelayLinePtr(&lbpef),
		lbpef64fGetParcorPtr(&lbpef),
		lbpef64fGetCoefPtr(&lbpef));

#ifndef STATIC
	lbpef64fResize(&lbpef,12);
#endif
	lbpef64fSetPcStepSize(&lbpef, 0.111f);
	lbpef64fSetFirFlag(&lbpef, 2);

	printf("L=%d, S1=%f,  F=%d, DLPtr=%d, PARCORPtr=%d, CofPtr=%d\n", 
		lbpef64fGetFilterLength(&lbpef),  
		lbpef64fGetPcStepSize(&lbpef), 
		lbpef64fGetFirFlag(&lbpef),
		lbpef64fGetDelayLinePtr(&lbpef),
		lbpef64fGetParcorPtr(&lbpef),
		lbpef64fGetCoefPtr(&lbpef));
#endif


#ifndef STATIC
	lbpef64fFree(&lbpef);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "LBPEF            %10.10f\n",  (double)duration/ CLOCKS_PER_SEC );

	return 0;
}


