/*  ********************************************************************
	TESTLMSLATTICE.C 
	simple system identification test application for the LMSLATTICE 
	adaptive algorithm.
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#include <time.h>
#include "asptlmslattice64f.h"
#include "asptfir64f.h"
#include "asptdisplay64f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED

#define ITERATIONS 5000
#define NF 8
#define NH 8

int main(int argc, char* argv[])
{

	asptLmslattice64f  lmsl ;	/* adaptive filter          */
	asptFir64f hFir;			/* unknown filter           */
	int		len = 8;	// filter length
	double	mu_p = 0.001f;
	double	mu_c = 0.01f;
	int		i,j=0, k=0;

	FILE	*outFile;			/* out signal log file	*/
	FILE	*errFile;			/* err signal log file	*/
	FILE	*inpFile;			/* inp signal log file	*/
	FILE	*desFile;			/* des signal log file	*/

	double	inpBuf[ITERATIONS];
	double	desBuf[ITERATIONS];
	double	errBuf[ITERATIONS];
	double	outBuf[ITERATIONS];

	double	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};
	double	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;		/* FIR coef. file */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	double	pCof[NF*2];		/* 2*filterLength */
	double	pDat[NF*3];		/* 3*filterLength */	
#endif

	/* create the unknown filter */
	i = fir64fInit(&hFir, NH, unknown);
	if (fir64fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset64f(inpBuf, ITERATIONS);
	reset64f(desBuf, ITERATIONS);
	reset64f(outBuf, ITERATIONS);
	reset64f(errBuf, ITERATIONS);	

	/* Create and initialize the asptLmslattice filter */
#ifndef STATIC
		i = lmslattice64fInit(&lmsl,len,mu_p, mu_c);
#else
		i = lmslattice64fInitStatic(&lmsl,len,mu_p, mu_c,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptLmslattice filter\n");
		exit(-1);
	}

	// make a random inp signal
	for (i=0; i<ITERATIONS; i++)
		inpBuf[i] = (double) rand()/(double)RAND_MAX - 0.5f;

	/* calculate the des signal */
	fir64fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

#ifdef FILTERONLY
	i = lmslattice64fSetCoef(&lmsl, unknown, 0, len, 0);
	printf("coef copied = %d\n",i);
	dispR64f(lmsl.coef,i);
	lmslattice64fSetParcorUpdateFlag(&lmsl,0);
#endif

	start = clock();
	duration = (double) 0.0;
	

	for (i=0; i<ITERATIONS;i++)	{

#ifdef FILTERONLY
		if (i < 1500){
			outBuf[i] = lmslattice64fFilterOnly(&lmsl, inpBuf[i]);
			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{	// filter and update the filter coefficient
			outBuf[i] = lmslattice64fFilterUpdate(&lmsl, inpBuf[i], desBuf[i]);
			errBuf[i] = lmslattice64fGetLastErrorSample(&lmsl);

		}
#ifdef RESET
		if (i == 3200){
			lmslattice64fReset(&lmsl);
		}
#endif
#ifdef RESIZE
#ifndef STATIC
		if (i == 3000)	{// resize
			len = 10;
			j = lmslattice64fResize(&lmsl, len);
			j = lmslattice64fGetCoef(&lmsl, fir, 0, len, 0);
			dispR64f(fir, j);
		}
#endif
#endif
		if (i==1500){
			lmslattice64fSetParcorUpdateFlag(&lmsl,0);
		}
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}
#ifndef SPEED
		/* write samples to streams */
		for (j=0; j<ITERATIONS;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
#endif
	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	// save and print the filter coefficients	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	i = lmslattice64fGetCoef(&lmsl, fir, 0, len, 0);
	fdispR64f(firFile,fir,len);
	dispR64f(fir,len);
	fclose(firFile);

#endif

#ifdef MACROS

	printf("L=%d, S1=%f, S2=%f, F=%d, DLPtr=%d, PARCORPtr=%d, CofPtr=%d\n", 
		lmslattice64fGetFilterLength(&lmsl),  
		lmslattice64fGetPcStepSize(&lmsl), 
		lmslattice64fGetLcStepSize(&lmsl),
		lmslattice64fGetParcorUpdateFlag(&lmsl),
		lmslattice64fGetDelayLinePtr(&lmsl),
		lmslattice64fGetParcorPtr(&lmsl),
		lmslattice64fGetCoefPtr(&lmsl));

#ifndef STATIC
	lmslattice64fResize(&lmsl,12);
#endif
	lmslattice64fSetPcStepSize(&lmsl, 0.111f);
	lmslattice64fSetLcStepSize(&lmsl, 2*0.111f);
	lmslattice64fSetParcorUpdateFlag(&lmsl, 2);

	printf("L=%d, S1=%f, S2=%f, F=%d, DLPtr=%d, PARCORPtr=%d, CofPtr=%d\n", 
		lmslattice64fGetFilterLength(&lmsl),  
		lmslattice64fGetPcStepSize(&lmsl), 
		lmslattice64fGetLcStepSize(&lmsl),
		lmslattice64fGetParcorUpdateFlag(&lmsl),
		lmslattice64fGetDelayLinePtr(&lmsl),
		lmslattice64fGetParcorPtr(&lmsl),
		lmslattice64fGetCoefPtr(&lmsl));
#endif

#ifndef STATIC
	lmslattice64fFree(&lmsl);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "LMSLATTICE       %10.10f\n",  (double)duration/ CLOCKS_PER_SEC );

	return 0;
}


